@@ -27,6 +27,8 @@ TextBlock::TextBlock (TextBuffer *buffer, int startLine)
   : m_buffer (buffer)
   , m_startLine (startLine)
 {
+  // reserve the block size
+  m_lines.reserve (m_buffer->m_blockSize);
 }
 
 TextBlock::~TextBlock ()
@@ -62,6 +64,16 @@ TextLine TextBlock::line (int line) const
   return m_lines.at(line);
 }
 
+void TextBlock::appendLine (const QString &textOfLine)
+{
+  m_lines.append (TextLine (new TextLineData(textOfLine)));
+}
+
+void TextBlock::clearLines ()
+{
+  m_lines.clear ();
+}
+    
 void TextBlock::text (QString &text) const
 {
   // combine all lines
@@ -74,7 +86,7 @@ void TextBlock::text (QString &text) const
   }
 }
 
-void TextBlock::wrapLine (const KTextEditor::Cursor &position)
+void TextBlock::wrapLine (const KTextEditor::Cursor &position, int fixStartLinesStartIndex)
 {
   // calc internal line
   int line = position.line () - startLine ();
@@ -111,6 +123,12 @@ void TextBlock::wrapLine (const KTextEditor::Cursor &position)
     m_lines.at(line)->markAsModified(true);
   }
 
+  /**
+   * fix all start lines
+   * we need to do this NOW, else the range update will FAIL!
+   * bug 313759
+   */
+  m_buffer->fixStartLines (fixStartLinesStartIndex);
 
   /**
    * notify the text history
@@ -168,7 +186,7 @@ void TextBlock::wrapLine (const KTextEditor::Cursor &position)
     range->checkValidity ();
 }
 
-void TextBlock::unwrapLine (int line, TextBlock *previousBlock)
+void TextBlock::unwrapLine (int line, TextBlock *previousBlock, int fixStartLinesStartIndex)
 {
   // calc internal line
   line = line - startLine ();
@@ -185,15 +203,12 @@ void TextBlock::unwrapLine (int line, TextBlock *previousBlock)
     TextLine newFirst = previousBlock->m_lines.last();
     m_lines[0] = newFirst;
     previousBlock->m_lines.erase (previousBlock->m_lines.begin() + (previousBlock->lines () - 1));
-
-    // append text
+    
     const int oldSizeOfPreviousLine = newFirst->text().size();
     if (oldFirst->length() > 0) {
+      // append text
       newFirst->textReadWrite().append (oldFirst->text());
-      QVector<int> tmp=newFirst->foldingListArray();
-      tmp+=oldFirst->foldingListArray();
-      newFirst->setFoldingList(tmp); //JOWENN MERGE CORRECTLY
-
+      
       // mark line as modified, since text was appended
       newFirst->markAsModified(true);
     }
@@ -201,6 +216,13 @@ void TextBlock::unwrapLine (int line, TextBlock *previousBlock)
     // patch startLine of this block
     --m_startLine;
 
+    /**
+     * fix all start lines
+     * we need to do this NOW, else the range update will FAIL!
+     * bug 313759
+     */
+    m_buffer->fixStartLines (fixStartLinesStartIndex);
+
     /**
      * notify the text history in advance
      */
@@ -210,9 +232,8 @@ void TextBlock::unwrapLine (int line, TextBlock *previousBlock)
      * cursor and range handling below
      */
 
-    // no cursors in this and previous block, no work to do..
-    // no need to touch ranges-cache, without cursors, no range could end between this blocks!
-    if (previousBlock->m_cursors.empty() && m_cursors.empty())
+    // no cursors in this block and the previous one, no work to do..
+    if (m_cursors.empty() && previousBlock->m_cursors.empty())
       return;
 
     // move all cursors because of the unwrapped line
@@ -223,38 +244,35 @@ void TextBlock::unwrapLine (int line, TextBlock *previousBlock)
         if (cursor->lineInBlock() == 0) {
           // patch column
           cursor->m_column += oldSizeOfPreviousLine;
-        }
 
-        // remember range, if any
-        if (cursor->kateRange())
-          changedRanges.insert (cursor->kateRange());
+          // remember range, if any
+          if (cursor->kateRange())
+            changedRanges.insert (cursor->kateRange());
+        }
     }
 
     // move cursors of the moved line from previous block to this block now
     QSet<TextCursor *> newPreviousCursors;
-    QSet<TextRange *> rangesMoved;
     foreach (TextCursor *cursor, previousBlock->m_cursors) {
       if (cursor->lineInBlock() == lastLineOfPreviousBlock) {
         cursor->m_line = 0;
         cursor->m_block = this;
         m_cursors.insert (cursor);
 
-        // remember ranges moved over block boundary
+        // remember range, if any
         if (cursor->kateRange())
-          rangesMoved.insert (cursor->kateRange());
+          changedRanges.insert (cursor->kateRange());
       }
       else
         newPreviousCursors.insert (cursor);
     }
     previousBlock->m_cursors = newPreviousCursors;
 
-    foreach (TextRange *range, rangesMoved) {
-        // either now only in new block
-        if (range->start().line () >= startLine())
-          previousBlock->removeRange (range);
-
-        // or now in both
+    // fixup the ranges that might be effected, because they moved from last line to this block
+    foreach (TextRange *range, changedRanges) {
+        // update both blocks
         updateRange (range);
+        previousBlock->updateRange (range);
     }
 
     // check validity of all ranges, might invalidate them...
@@ -268,13 +286,8 @@ void TextBlock::unwrapLine (int line, TextBlock *previousBlock)
   // easy: just move text to previous line and remove current one
   const int oldSizeOfPreviousLine = m_lines.at(line-1)->length();
   const int sizeOfCurrentLine = m_lines.at(line)->length();
-  if (sizeOfCurrentLine > 0) {
+  if (sizeOfCurrentLine > 0)
     m_lines.at(line-1)->textReadWrite().append (m_lines.at(line)->text());
-    QVector<int> tmp=m_lines.at(line-1)->foldingListArray();
-    tmp+=m_lines.at(line)->foldingListArray();
-    m_lines.at(line-1)->setFoldingList(tmp); //JOWENN MERGE CORRECTLY
-    
-  }
 
   const bool lineChanged = (oldSizeOfPreviousLine > 0 && m_lines.at(line - 1)->markedAsModified())
     || (sizeOfCurrentLine > 0 && (oldSizeOfPreviousLine > 0 || m_lines.at(line)->markedAsModified()));
@@ -284,6 +297,13 @@ void TextBlock::unwrapLine (int line, TextBlock *previousBlock)
 
   m_lines.erase (m_lines.begin () + line);
 
+  /**
+   * fix all start lines
+   * we need to do this NOW, else the range update will FAIL!
+   * bug 313759
+   */
+  m_buffer->fixStartLines (fixStartLinesStartIndex);
+
   /**
    * notify the text history in advance
    */
@@ -371,7 +391,7 @@ void TextBlock::insertText (const KTextEditor::Cursor &position, const QString &
       // patch column of cursor
       if (cursor->m_column <= oldLength)
         cursor->m_column += text.size ();
-      
+
       // special handling if cursor behind the real line, e.g. non-wrapping cursor in block selection mode
       else if (cursor->m_column < textOfLine.size())
         cursor->m_column = textOfLine.size();
@@ -473,11 +493,7 @@ TextBlock *TextBlock::splitBlock (int fromLine)
 
   // move cursors
   QSet<TextCursor*> oldBlockSet;
-  QSet<TextRange*> rangesInteresting;
   foreach (TextCursor *cursor, m_cursors) {
-      if (cursor->kateRange())
-        rangesInteresting.insert (cursor->kateRange());
-
       if (cursor->lineInBlock() >= fromLine) {
         cursor->m_line = cursor->lineInBlock() - fromLine;
         cursor->m_block = newBlock;
@@ -488,12 +504,12 @@ TextBlock *TextBlock::splitBlock (int fromLine)
   }
   m_cursors = oldBlockSet;
 
-  foreach (TextRange *range, rangesInteresting) {
-      // only in new block
-      if (range->start().line () >= newBlock->startLine()) {
-        removeRange (range);
-        newBlock->updateRange (range);
-      }
+  // fix ALL ranges!
+  QList<TextRange*> allRanges = m_uncachedRanges.toList() + m_cachedLineForRanges.keys();
+  foreach (TextRange *range, allRanges) {
+      // update both blocks
+      updateRange (range);
+      newBlock->updateRange (range);
   }
 
   // return the new generated block
@@ -516,10 +532,12 @@ void TextBlock::mergeBlock (TextBlock *targetBlock)
     targetBlock->m_lines.append (m_lines.at(i));
   m_lines.clear ();
 
+  // fix ALL ranges!
   QList<TextRange*> allRanges = m_uncachedRanges.toList() + m_cachedLineForRanges.keys();
   foreach(TextRange* range, allRanges) {
-    removeRange(range);
-    targetBlock->updateRange(range);
+    // update both blocks
+    updateRange (range);
+    targetBlock->updateRange (range);
   }
 }
 
@@ -571,13 +589,21 @@ void TextBlock::updateRange (TextRange* range)
   const int startLine = range->startInternal().lineInternal();
   const int endLine = range->endInternal().lineInternal();
   const bool isSingleLine = startLine == endLine;
-  
+
+  /**
+   * perhaps remove range and be done
+   */
+  if ((endLine < m_startLine) || (startLine >= (m_startLine + lines()))) {
+    removeRange (range);
+    return;
+  }
+
   /**
    * The range is still a single-line range, and is still cached to the correct line.
    */
   if(isSingleLine && m_cachedLineForRanges.contains (range) && (m_cachedLineForRanges.value(range) == startLine - m_startLine))
     return;
-  
+
   /**
    * The range is still a multi-line range, and is already in the correct set.
    */
@@ -587,9 +613,8 @@ void TextBlock::updateRange (TextRange* range)
   /**
    * remove, if already there!
    */
-  if(containsRange(range))
-    removeRange(range);
-  
+  removeRange(range);
+
   /**
    * simple case: multi-line range
    */
@@ -600,18 +625,18 @@ void TextBlock::updateRange (TextRange* range)
     m_uncachedRanges.insert(range);
     return;
   }
-    
+
   /**
-   * The range is contained by a single line, put it into the line-cache 
+   * The range is contained by a single line, put it into the line-cache
    */
   const int lineOffset = startLine - m_startLine;
-  
+
   /**
    * enlarge cache if needed
    */
   if (m_cachedRangesForLine.size() <= lineOffset)
     m_cachedRangesForLine.resize(lineOffset+1);
-    
+
   /**
    * insert into mapping
    */
@@ -629,10 +654,9 @@ void TextBlock::removeRange (TextRange* range)
      * must be only uncached!
      */
     Q_ASSERT (!m_cachedLineForRanges.contains(range));
-
     return;
   }
-  
+
   /**
    * cached range?
    */
@@ -642,12 +666,12 @@ void TextBlock::removeRange (TextRange* range)
      * must be only cached!
      */
     Q_ASSERT (!m_uncachedRanges.contains(range));
-    
+
     /**
      * query the range from cache, must be there
      */
     Q_ASSERT (m_cachedRangesForLine.at(*it).contains(range));
-    
+
     /**
      * remove it and be done
      */
@@ -655,9 +679,9 @@ void TextBlock::removeRange (TextRange* range)
     m_cachedLineForRanges.erase(it);
     return;
   }
-  
+
   /**
-   * else: perhaps later see if we can assert, that this is not allowed?
+   * else: range was not for this block, just do nothing, removeRange should be "safe" to use
    */
 }
 
