@@ -1151,6 +1151,7 @@ void KateCompletionWidget::switchList() {
       if( m_argumentHintModel->rowCount(QModelIndex()) != 0 ) {
         m_entryList->setCurrentIndex(QModelIndex());
         m_argumentHintTree->setCurrentIndex(m_argumentHintModel->index(m_argumentHintModel->rowCount(QModelIndex())-1, 0));
+        m_inCompletionList = false;
       }
   } else {
       if( m_presentationModel->rowCount(QModelIndex()) != 0 ) {
@@ -1158,9 +1159,9 @@ void KateCompletionWidget::switchList() {
         m_entryList->setCurrentIndex(m_presentationModel->index(0, 0));
         if(model()->hasGroups()) //If we have groups we have to move on, because the first item is a label
           m_entryList->nextCompletion();
+        m_inCompletionList = true;
       }
   }
-  m_inCompletionList = !m_inCompletionList;
 }
 
 void KateCompletionWidget::showConfig( )
@@ -1286,42 +1287,6 @@ void KateCompletionWidget::removeText (const KTextEditor::Range &)
   m_automaticInvocationTimer->stop();
 }
 
-#if 0
-void KateCompletionWidget::editDone(KateEditInfo * edit)
-{
-  return;
-  
-  if(!edit->newText().join("\n").trimmed().isEmpty())
-  m_lastInsertionByUser = edit->editSource() == Kate::UserInputEdit;
-
-  if (!view()->config()->automaticCompletionInvocation()
-       || (edit->editSource() != Kate::UserInputEdit)
-       || edit->isRemoval()
-       || (edit->editSource() != Kate::UserInputEdit && edit->editSource() != Kate::CodeCompletionEdit)
-       || edit->newText().isEmpty() )
-  {
-    m_automaticInvocationLine.clear();
-    m_automaticInvocationTimer->stop();
-    return;
-  }
-
-  if(m_automaticInvocationAt != edit->newRange().start()) {
-    m_automaticInvocationLine.clear();
-    m_lastInsertionByUser = edit->editSource() == Kate::UserInputEdit;
-  }
-
-  m_automaticInvocationLine += edit->newText().last();
-  m_automaticInvocationAt = edit->newRange().end();
-
-  if (m_automaticInvocationLine.isEmpty()) {
-    m_automaticInvocationTimer->stop();
-    return;
-  }
-
-  m_automaticInvocationTimer->start(m_automaticInvocationDelay);
-}
-#endif
-
 void KateCompletionWidget::automaticInvocation()
 {
   //kDebug()<<"m_automaticInvocationAt:"<<m_automaticInvocationAt;
