@@ -45,8 +45,9 @@ static const QChar tabChar('\t');
 static const QChar spaceChar(' ');
 static const QChar nbSpaceChar(0xa0); // non-breaking space
 
-KateRenderer::KateRenderer(KateDocument* doc, KateView *view)
+KateRenderer::KateRenderer(KateDocument* doc, Kate::TextFolding &folding, KateView *view)
   : m_doc(doc)
+  , m_folding (folding)
     , m_view (view)
     , m_tabWidth(m_doc->config()->tabWidth())
     , m_indentWidth(m_doc->config()->indentationWidth())
@@ -336,10 +337,10 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine( const Kate::Te
 
     // Add the inbuilt highlighting to the list
     NormalRenderRange* inbuiltHighlight = new NormalRenderRange();
-    const QVector<int> &al = textLine->attributesList();
-    for (int i = 0; i+2 < al.count(); i += 3) {
-      inbuiltHighlight->addRange(new KTextEditor::Range(KTextEditor::Cursor(line, al[i]), al[i+1]), specificAttribute(al[i+2]));
-    }
+    const QVector<Kate::TextLineData::Attribute> &al = textLine->attributesList();
+    for (int i = 0; i < al.count(); ++i)
+      if (al[i].length > 0 && al[i].attributeValue > 0)
+        inbuiltHighlight->addRange(new KTextEditor::Range(KTextEditor::Cursor(line, al[i].offset), al[i].length), specificAttribute(al[i].attributeValue));
     renderRanges.append(inbuiltHighlight);
 
     if (!completionHighlight) {
@@ -624,30 +625,29 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
         nbSpaceIndex = text.indexOf(nbSpaceChar, nbSpaceIndex + 1);
       }
 
-      // Draw tab stops and trailing spaces
-      if (showTabs() || showTrailingSpaces()) {
-        if (showTabs()) {
-          int tabIndex = text.indexOf(tabChar, line.lineLayout().xToCursor(xStart));
-          while (tabIndex != -1 && tabIndex < line.endCol()) {
-            int x = line.lineLayout().cursorToX(tabIndex);
-            if (x > xEnd)
-              break;
-            paintTabstop(paint, x - xStart + spaceWidth()/2.0, y);
-            tabIndex = text.indexOf(tabChar, tabIndex + 1);
-          }
+      // draw tab stop indicators
+      if (showTabs()) {
+        int tabIndex = text.indexOf(tabChar, line.lineLayout().xToCursor(xStart));
+        while (tabIndex != -1 && tabIndex < line.endCol()) {
+          int x = line.lineLayout().cursorToX(tabIndex);
+          if (x > xEnd)
+            break;
+          paintTabstop(paint, x - xStart + spaceWidth()/2.0, y);
+          tabIndex = text.indexOf(tabChar, tabIndex + 1);
         }
+      }
 
-        if (showTrailingSpaces()) {
-          int spaceIndex = line.endCol() - 1;
-          int trailingPos = range->textLine()->lastChar();
-          if (trailingPos < 0)
-            trailingPos = 0;
-          if (spaceIndex >= trailingPos) {
-            while (spaceIndex >= line.startCol() && text.at(spaceIndex).isSpace()) {
-              if (text.at(spaceIndex) != '\t' || !showTabs())
-                paintTrailingSpace(paint, line.lineLayout().cursorToX(spaceIndex) - xStart + spaceWidth()/2.0, y);
-              --spaceIndex;
-            }
+      // draw trailing spaces
+      if (showTrailingSpaces()) {
+        int spaceIndex = line.endCol() - 1;
+        int trailingPos = range->textLine()->lastChar();
+        if (trailingPos < 0)
+          trailingPos = 0;
+        if (spaceIndex >= trailingPos) {
+          while (spaceIndex >= line.startCol() && text.at(spaceIndex).isSpace()) {
+            if (text.at(spaceIndex) != '\t' || !showTabs())
+              paintTrailingSpace(paint, line.lineLayout().cursorToX(spaceIndex) - xStart + spaceWidth()/2.0, y);
+            --spaceIndex;
           }
         }
       }
