@@ -29,7 +29,6 @@
 #include <ktexteditor/movingrange.h>
 #include <ktexteditor/containerinterface.h>
 #include <ktexteditor/highlightinterface.h>
-#include "katecodefolding.h"
 #include "kateconfig.h"
 #include "katedocument.h"
 #include <katebuffer.h>
@@ -134,7 +133,7 @@ void KateScrollBar::setShowMiniMap(bool b)
     connect(m_doc, SIGNAL(textChanged(KTextEditor::Document*)), &m_updateTimer, SLOT(start()), Qt::UniqueConnection);
     connect(m_view, SIGNAL(delayedUpdateOfView()), &m_updateTimer, SLOT(start()), Qt::UniqueConnection);
     connect(&m_updateTimer, SIGNAL(timeout()), this, SLOT(updatePixmap()), Qt::UniqueConnection);
-    connect(m_doc->foldingTree(), SIGNAL(regionVisibilityChanged()), &m_updateTimer, SLOT(start()), Qt::UniqueConnection);
+    connect(&(m_view->textFolding()), SIGNAL(foldingRangesChanged()), &m_updateTimer, SLOT(start()), Qt::UniqueConnection);
   }
   else if (!b) {
     disconnect(&m_updateTimer);
@@ -204,7 +203,7 @@ void KateScrollBar::mousePressEvent(QMouseEvent* e)
   m_toolTipPos = e->globalPos() - QPoint(e->pos().x(), 0);
   const int fromLine = m_viewInternal->toRealCursor(m_viewInternal->startPos()).line() + 1;
   const int lastLine = m_viewInternal->toRealCursor(m_viewInternal->endPos()).line() + 1;
-  QToolTip::showText(m_toolTipPos, i18nc("from line - to line", "<center>%1<br>&mdash;<br/>%2</center>", fromLine, lastLine), this);
+  QToolTip::showText(m_toolTipPos, i18nc("from line - to line", "<center>%1<br/>&#x2014;<br/>%2</center>", fromLine, lastLine), this);
 
   redrawMarks();
 }
@@ -252,7 +251,7 @@ void KateScrollBar::mouseMoveEvent(QMouseEvent* e)
     m_toolTipPos = e->globalPos() - QPoint(e->pos().x(), 0);
     const int fromLine = m_viewInternal->toRealCursor(m_viewInternal->startPos()).line() + 1;
     const int lastLine = m_viewInternal->toRealCursor(m_viewInternal->endPos()).line() + 1;
-    QToolTip::showText(m_toolTipPos, i18nc("from line - to line", "<center>%1<br>&mdash;<br/>%2</center>", fromLine, lastLine), this);
+    QToolTip::showText(m_toolTipPos, i18nc("from line - to line", "<center>%1<br/>&#x2014;<br/>%2</center>", fromLine, lastLine), this);
   }
 }
 
@@ -267,7 +266,7 @@ void KateScrollBar::paintEvent(QPaintEvent *e)
 }
 
 // This function is optimized for bing called in sequence.
-const QColor KateScrollBar::charColor(const QVector<int> &attributes, int &attributeIndex,
+const QColor KateScrollBar::charColor(const QVector<Kate::TextLineData::Attribute> &attributes, int &attributeIndex,
                                       const QList<QTextLayout::FormatRange> &decorations,
                                       const QColor &defaultColor, int x, QChar ch)
 {
@@ -297,12 +296,12 @@ const QColor KateScrollBar::charColor(const QVector<int> &attributes, int &attri
   if (!styleFound) {
     // go to the block containing x
     while ((attributeIndex < attributes.size()) &&
-      ((attributes[attributeIndex] + attributes[attributeIndex+1]) < x))
+      ((attributes[attributeIndex].offset + attributes[attributeIndex].length) < x))
     {
-      attributeIndex += 3;
+      ++attributeIndex;
     }
-    if ((attributeIndex < attributes.size()) && (x < attributes[attributeIndex] + attributes[attributeIndex+1])) {
-      color = m_view->renderer()->attribute(attributes[attributeIndex+2])->foreground().color();
+    if ((attributeIndex < attributes.size()) && (x < attributes[attributeIndex].offset + attributes[attributeIndex].length)) {
+      color = m_view->renderer()->attribute(attributes[attributeIndex].attributeValue)->foreground().color();
     }
   }
 
@@ -328,7 +327,7 @@ void KateScrollBar::updatePixmap()
 
   // For performance reason, only every n-th line will be drawn if the widget is
   // sufficiently small compared to the amount of lines in the document.
-  int docLineCount = m_doc->visibleLines();
+  int docLineCount = m_view->textFolding().visibleLines();
   int pixmapLineCount = docLineCount;
   if (m_view->config()->scrollPastEnd()) {
     pixmapLineCount += pageStep();
@@ -352,7 +351,7 @@ void KateScrollBar::updatePixmap()
   int pixmapLineWidth = s_pixelMargin + s_lineWidth/charIncrement;
 
   //kDebug(13040) << "l" << lineIncrement << "c" << charIncrement << "d" << lineDivisor;
-  //kDebug(13040) << "pixmap" << pixmapLineCount << pixmapLineWidth << "docLines" << m_doc->visibleLines() << "height" << m_grooveHeight;
+  //kDebug(13040) << "pixmap" << pixmapLineCount << pixmapLineWidth << "docLines" << m_view->textFolding().visibleLines() << "height" << m_grooveHeight;
 
   QColor backgroundColor;
   QColor defaultTextColor;
@@ -387,7 +386,7 @@ void KateScrollBar::updatePixmap()
     // Iterate over all visible lines, drawing them.
     for (int virtualLine=0; virtualLine < docLineCount; virtualLine += lineIncrement) {
 
-      int realLineNumber = m_doc->getRealLine(virtualLine);
+      int realLineNumber = m_view->textFolding().visibleLineToLine(virtualLine);
       QString lineText = m_doc->line(realLineNumber);
 
       // use this to control the offset of the text from the left
@@ -398,7 +397,7 @@ void KateScrollBar::updatePixmap()
       }
       const Kate::TextLine& kateline = m_doc->plainKateTextLine(realLineNumber);
 
-      QVector<int> attributes = kateline->attributesList();
+      const QVector<Kate::TextLineData::Attribute> &attributes = kateline->attributesList();
       QList< QTextLayout::FormatRange > decorations = m_view->renderer()->decorationsForLine(kateline, realLineNumber);
       int attributeIndex = 0;
 
@@ -454,7 +453,7 @@ void KateScrollBar::updatePixmap()
     // since it requires querying every line.
     if ( m_doc->lines() < 50000 ) {
       for ( int lineno = 0; lineno < docLineCount; lineno++ ) {
-        int realLineNo = m_doc->getRealLine(lineno);
+        int realLineNo = m_view->textFolding().visibleLineToLine(lineno);
         const Kate::TextLine& line = m_doc->plainKateTextLine(realLineNo);
         if ( line->markedAsModified() ) {
           painter.setPen(modifiedLineColor);
@@ -531,6 +530,7 @@ void KateScrollBar::miniMapPaintEvent(QPaintEvent *e)
   // calculate colors
   QColor backgroundColor;
   QColor foregroundColor;
+  // TODO KDE5: If HighlightInterface is a View interface, use HighlightInterface::defaultStyle() again.
   if (m_doc->defaultStyle(KTextEditor::HighlightInterface::dsNormal)) {
     backgroundColor = m_doc->defaultStyle(KTextEditor::HighlightInterface::dsNormal)->background().color();
     foregroundColor = m_doc->defaultStyle(KTextEditor::HighlightInterface::dsNormal)->foreground().color();
@@ -705,7 +705,7 @@ void KateScrollBar::sliderChange ( SliderChange change )
   if (m_leftMouseDown || m_middleMouseDown) {
     const int fromLine = m_viewInternal->toRealCursor(m_viewInternal->startPos()).line() + 1;
     const int lastLine = m_viewInternal->toRealCursor(m_viewInternal->endPos()).line() + 1;
-    QToolTip::showText(m_toolTipPos, i18nc("from line - to line", "<center>%1<br>&mdash;<br/>%2</center>", fromLine, lastLine), this);
+    QToolTip::showText(m_toolTipPos, i18nc("from line - to line", "<center>%1<br/>&#x2014;<br/>%2</center>", fromLine, lastLine), this);
   }
 }
 
@@ -723,7 +723,7 @@ void KateScrollBar::redrawMarks()
 void KateScrollBar::recomputeMarksPositions()
 {
   m_lines.clear();
-  int visibleLines = m_doc->visibleLines();
+  int visibleLines = m_view->textFolding().visibleLines();
 
   QStyleOptionSlider opt;
   initStyleOption(&opt);
@@ -732,7 +732,6 @@ void KateScrollBar::recomputeMarksPositions()
   int realHeight = style()->subControlRect(QStyle::CC_ScrollBar, &opt, QStyle::SC_ScrollBarAddPage, this).bottom() - topMargin - 1;
 
   const QHash<int, KTextEditor::Mark*> &marks = m_doc->marks();
-  KateCodeFoldingTree *tree = m_doc->foldingTree();
 
   for (QHash<int, KTextEditor::Mark*>::const_iterator i = marks.constBegin(); i != marks.constEnd(); ++i)
   {
@@ -740,19 +739,7 @@ void KateScrollBar::recomputeMarksPositions()
 
     uint line = mark->line;
 
-    if (tree)
-    {
-      KateCodeFoldingNode *node = tree->findNodeForLine(line);
-
-      while (node)
-      {
-        if (!node->isVisible())
-          line = tree->getStartLine(node);
-        node = node->getParentNode();
-      }
-    }
-
-    line = m_doc->getVirtualLine(line);
+    line = m_view->textFolding().lineToVisibleLine(line);
 
     double d = (double)line / (visibleLines - 1);
     m_lines.insert(topMargin + (int)(d * realHeight),
@@ -895,7 +882,6 @@ KateCmdLineEdit::KateCmdLineEdit (KateCommandLineBar *bar, KateView *view)
 void KateCmdLineEdit::hideEvent(QHideEvent *e)
 {
   Q_UNUSED(e);
-  m_view->showViModeBar();
 }
 
 
@@ -1636,15 +1622,6 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
   p.setRenderHints (QPainter::TextAntialiasing);
   p.setFont ( m_view->renderer()->config()->font() ); // for line numbers
 
-  KateLineInfo oldInfo;
-  if (startz < lineRangesSize)
-  {
-    if ((m_viewInternal->cache()->viewLine(startz).line()-1) < 0)
-      oldInfo.topLevel = true;
-    else
-      m_doc->lineInfo(&oldInfo,m_viewInternal->cache()->viewLine(startz).line()-1);
-  }
-
   KTextEditor::AnnotationModel *model = m_view->annotationModel() ?
       m_view->annotationModel() : m_doc->annotationModel();
 
@@ -1794,17 +1771,13 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
     }
 
     // folding markers
-    if( m_foldingMarkersOn )
+    if( m_foldingMarkersOn)
     {
-      if( realLine > -1 )
-      {
-        KateLineInfo info;
-        m_doc->lineInfo(&info,realLine);
-
-        // first icon border background
-        p.fillRect(lnX, y, iconPaneWidth, h, m_view->renderer()->config()->iconBarColor());
-        // ... with possible additional folding highlighting
-        if (m_foldingRange && m_foldingRange->overlapsLine (realLine)) {
+      // first icon border background
+      p.fillRect(lnX, y, iconPaneWidth, h, m_view->renderer()->config()->iconBarColor());
+        
+      // possible additional folding highlighting
+      if ((realLine >= 0) && m_foldingRange && m_foldingRange->overlapsLine (realLine)) {
           p.save();
 
           // use linear gradient as brush
@@ -1832,28 +1805,29 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
             p.drawRect(lnX, y-r, iconPaneWidth, h+2*r);
           }
           p.restore();
-        }
+      }
+          
+      if ((realLine >= 0) && (m_viewInternal->cache()->viewLine(z).startCol() == 0))
+      {
+        QVector<QPair<qint64, Kate::TextFolding::FoldingRangeFlags> > startingRanges = m_view->textFolding().foldingRangesStartingOnLine (realLine);
+        bool anyFolded = false;
+        for (int i = 0; i < startingRanges.size(); ++i)
+          if (startingRanges[i].second & Kate::TextFolding::Folded)
+            anyFolded = true;
 
-        if (!info.topLevel)
+        Kate::TextLine tl = m_doc->kateTextLine(realLine);
+
+        if (!startingRanges.isEmpty() || tl->markedAsFoldingStart())
         {
-          if (info.startsInVisibleBlock && m_viewInternal->cache()->viewLine(z).startCol() == 0)
+          if (anyFolded)
           {
             paintTriangle (p, m_view->renderer()->config()->foldingColor(), lnX, y, iconPaneWidth, h, false);
           }
-          else if (info.startsVisibleBlock && (m_viewInternal->cache()->viewLine(z).startCol() == 0))
+          else
           {
             paintTriangle (p, m_view->renderer()->config()->foldingColor(), lnX, y, iconPaneWidth, h, true);
           }
-          else
-          {
-           // p.drawLine(lnX+halfIPW,y,lnX+halfIPW,y+h-1);
-
-           // if (info.endsBlock && !m_viewInternal->cache()->viewLine(z).wrap())
-            //  p.drawLine(lnX+halfIPW,y+h-1,lnX+iconPaneWidth-2,y+h-1);
-          }
         }
-
-        oldInfo = info;
       }
 
       lnX += iconPaneWidth;
@@ -1948,31 +1922,27 @@ void KateIconBorder::showBlock()
 {
   if (m_nextHighlightBlock == m_currentBlockLine) return;
   m_currentBlockLine = m_nextHighlightBlock;
-
-  // get the new range, that should be highlighted
+  
+  /**
+   * compute to which folding range we belong
+   * FIXME: optimize + perhaps have some better threshold or use timers!
+   */
   KTextEditor::Range newRange = KTextEditor::Range::invalid();
-  KateCodeFoldingTree *tree = m_doc->foldingTree();
-  if (tree) {
-    KateCodeFoldingNode *node = tree->findNodeForLine(m_currentBlockLine);
-    KTextEditor::Cursor beg;
-    KTextEditor::Cursor end;
-    if (node != tree->rootNode () && node->getBegin(tree, &beg)) {
-      if (node->getEnd(tree, &end))
-        newRange = KTextEditor::Range(beg, end);
-      else
-        newRange = KTextEditor::Range(beg, m_viewInternal->doc()->documentEnd());
+  for (int line = m_currentBlockLine; line >= qMax(0, m_currentBlockLine-1024); --line) {
+    /**
+     * try if we have folding range from that line, should be fast per call
+     */
+    KTextEditor::Range foldingRange = m_doc->buffer().computeFoldingRangeForStartLine (line);
+    if (!foldingRange.isValid())
+      continue;
+    
+    /**
+     * does the range reach us?
+     */
+    if (foldingRange.overlapsLine (m_currentBlockLine)) {
+      newRange = foldingRange;
+      break;
     }
-    KateLineInfo info;
-    tree->getLineInfo(&info, m_currentBlockLine);
-    if ((info.startsVisibleBlock)) {
-      node=tree->findNodeStartingAt(m_currentBlockLine);
-      if (node) {
-        if (node != tree->rootNode () && node->getBegin(tree, &beg) && node->getEnd(tree, &end)) {
-          newRange = KTextEditor::Range(beg, end);
-        }
-      }
-    }
-
   }
 
   if (newRange.isValid() && m_foldingRange && *m_foldingRange == newRange) {
@@ -2098,16 +2068,26 @@ void KateIconBorder::mouseReleaseEvent( QMouseEvent* e )
     }
 
     if ( area == FoldingMarkers) {
-      // if a folding range exists, fold this one, -> use start line of the range
-      int lineToFold = cursorOnLine;
-      if (m_foldingRange && m_foldingRange->start().line() >= 0) {
-        lineToFold = m_foldingRange->start().line();
-      }
-      KateLineInfo info;
-      m_doc->lineInfo(&info, lineToFold);
-      if ((info.startsVisibleBlock) || (info.startsInVisibleBlock)) {
-        emit toggleRegionVisibility(lineToFold);
-      }
+        // ask the folding info for this line, if any folds are around!
+        QVector<QPair<qint64, Kate::TextFolding::FoldingRangeFlags> > startingRanges = m_view->textFolding().foldingRangesStartingOnLine (cursorOnLine);
+        bool anyFolded = false;
+        for (int i = 0; i < startingRanges.size(); ++i)
+          if (startingRanges[i].second & Kate::TextFolding::Folded)
+            anyFolded = true;
+        
+        // fold or unfold all ranges, remember if any action happened!
+        bool actionDone = false;
+        for (int i = 0; i < startingRanges.size(); ++i)
+          actionDone = (anyFolded ? m_view->textFolding().unfoldRange (startingRanges[i].first) : m_view->textFolding().foldRange (startingRanges[i].first)) || actionDone;
+        
+        // if no action done, try to fold it, create non-persistent folded range, if possible!
+        if (!actionDone) {
+          // either use the fold for this line or the range that is highlighted ATM if any!
+          KTextEditor::Range foldingRange = m_view->doc()->buffer().computeFoldingRangeForStartLine (cursorOnLine);
+          if (!foldingRange.isValid() && m_foldingRange)
+            foldingRange = m_foldingRange->toRange ();
+          m_view->textFolding().newFoldingRange (foldingRange, Kate::TextFolding::Folded);
+        }
     }
 
     if ( area == AnnotationBorder ) {
