@@ -22,7 +22,6 @@
 #define __KATE_VIEW_HELPERS_H__
 
 #include <kselectaction.h>
-#include <kencodingprober.h>
 #include <klineedit.h>
 #include <KActionMenu>
 
@@ -38,6 +37,7 @@
 #include <ktexteditor/containerinterface.h>
 #include <ktexteditor/cursor.h>
 #include "katepartprivate_export.h"
+#include "katetextline.h"
 
 class KateDocument;
 class KateView;
@@ -114,7 +114,7 @@ Q_SIGNALS:
 
     int minimapYToStdY(int y);
 
-    const QColor charColor(const QVector<int> &attributes, int &attributeIndex,
+    const QColor charColor(const QVector<Kate::TextLineData::Attribute> &attributes, int &attributeIndex,
                            const QList<QTextLayout::FormatRange> &decorations,
                            const QColor &defaultColor, int x, QChar ch);
 
@@ -179,8 +179,6 @@ class KateIconBorder : public QWidget
     enum BorderArea { None, LineNumbers, IconBorder, FoldingMarkers, AnnotationBorder, ModificationBorder };
     BorderArea positionToArea( const QPoint& ) const;
 
-  Q_SIGNALS:
-    void toggleRegionVisibility( int );
   public Q_SLOTS:
     void updateAnnotationBorderWidth();
     void updateAnnotationLine( int line );
