@@ -51,6 +51,8 @@ class KateViVisualMode : public KateViNormalMode {
     void setStart( const Cursor& c ) { m_start = c; }
     Cursor getStart() { return m_start; }
 
+    void goToPos( const Cursor& c);
+
     ViMode getLastVisualMode() const { return m_lastVisualMode; }
     Cursor getStart() const { return m_start; }
 
@@ -84,7 +86,7 @@ public Q_SLOTS:
     ViMode m_mode;
     Cursor m_start;
     ViMode m_lastVisualMode; // used when reselecting a visual selection
-    bool m_selection_is_changed_inside_ViMode;
+    bool m_selection_is_changed_inside_ViMode; // TODO - check whether this is still needed now that we have m_commandBeingExecuted.
 };
 
 #endif
